﻿using ProjectWarmlyShip.CollectionGenericObjects;
using ProjectWarmlyShip.Drawnings;

namespace ProjectWarmlyShip;

public partial class FormShipCollection : Form
{
    private AbstractCompany? _company = null;
    public FormShipCollection()
    {
        InitializeComponent();
    }
    private void comboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new ShipPortService(pictureBox.Width, pictureBox.Height, new MassiveGenericObjects<DrawningShip>());
                break;
        }
    }
    private void CreateObject(string type)
    {
        if (_company == null)
        {
            return;
        }
        Random random = new();
        DrawningShip drawningShip;
        switch (type)
        {
            case nameof(DrawningShip):
                drawningShip = new DrawningShip(random.Next(100, 300),
                random.Next(1000, 3000), GetColor(random));
                break;
            case nameof(DrawningWarmlyShip):
                // TODO вызов диалогового окна для выбора цвета (made)
                drawningShip = new DrawningWarmlyShip(random.Next(100, 300), random.Next(1000, 3000),
                GetColor(random), GetColor(random),
                Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }
        if (_company + drawningShip != -1)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось добавить объект");
        }
    }
    private static Color GetColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }
        return color;
    }

    private void buttonAddShip_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawningShip));
    }
    private void buttonAddWarmlyShip_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawningWarmlyShip));
    }
    private void buttonRemoveShip_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
        {
            return;
        }
        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        int pos = Convert.ToInt32(maskedTextBox.Text);
        if (_company - pos != null)
        {
            MessageBox.Show("Объект удален");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось удалить объект");
        }
    }
    private void buttonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        DrawningShip? ship = null;
        int counter = 100;
        while (ship == null)
        {
            ship = _company.GetRandomObject();
            counter--;
            if (counter <= 0)
            {
                break;
            }
        }
        if (ship == null)
        {
            return;
        }
        FormWarmlyShip form = new()
        {
            SetShip = ship
        };
        form.ShowDialog();
    }
    private void buttonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        pictureBox.Image = _company.Show();
    }
}
